"""Convert uploaded PDF to JPG.

Credit: Pierre Sutra (https://github.com/otrack/cloud-computing-infrastructure
/tree/master/scaling/pdfmagic).
"""

import os
import socket
import sys
import uuid
from subprocess import CalledProcessError
from subprocess import run as sp_run

import web

URLS = ('/', 'Upload', '/Download/(.+)', 'Download')

FILESDIR = '/tmp'

HOST = str(socket.gethostbyname(socket.gethostname()))


class Download:
    #pylint: disable=too-few-public-methods, invalid-name, no-self-use

    def GET(self, fileid):
        path = FILESDIR + '/' + str(fileid) + '.pdf'
        web.header('Content-Disposition',
                   'attachment; filename="' + str(fileid) + '.pdf"')
        web.header('Content-type', 'application/pdf')
        web.header('Content-transfer-encoding', 'binary')
        return open(path, 'rb').read()


class Upload:
    #pylint: disable=too-few-public-methods, invalid-name, no-self-use

    GET_FORM = """
<html><head></head><body>welcome @""" + HOST + """; please upload your file:
<form method="POST" enctype="multipart/form-data" action="">
<input type="file" name="myfile" />
<br/>
<input type="submit" />
</form>
</body></html
"""

    def GET(self):
        web.header("Content-Type", "text/html; charset=utf-8")
        return Upload.GET_FORM

    def POST(self):
        req = web.input(myfile={})

        if 'myfile' in req:
            fileid = str(uuid.uuid4())
            path = os.path.join(FILESDIR, str(fileid))

            with open(path + '.jpg', 'wb') as fout:
                fout.write(req.myfile.file.read())

            try:
                sp_run(['convert', path + '.jpg', path + '.pdf'],
                       check=True,
                       capture_output=True,
                       text=True)
            except CalledProcessError as err:
                print(
                    f'Calling ImageMagick\'s `convert` failed with error code '
                    f'{err.returncode}',
                    file=sys.stderr)
                print('stdout:\n' + err.stdout, file=sys.stderr)
                print('stderr:\n' + err.stderr, file=sys.stderr)

            raise web.seeother('/Download/' + fileid)


if __name__ == "__main__":
    app = web.application(URLS, globals())
    app.run()
